import importlib.util
import os

import web3
from eth_account import Account
from eth_account.messages import encode_defunct


def get_test_by_name(name, path=''):
    """
    """
    spec = importlib.util.spec_from_file_location(f"{name}_test", os.path.join(path, f"{name}_test.py"))
    module = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(module)
    return module.test_contract


def get_sign(a, b, c, i, sender=None):
    msg_hash = web3.Web3.solidityKeccak(["address"], [a.address])
    if sender:
        data = sender.sign_message(encode_defunct(msg_hash)).signature
    else:
        data = a.sign_message(encode_defunct(msg_hash)).signature

    return data


def get_sign_(a, sender):
    msg_hash = web3.Web3.solidityKeccak(["address"], [a])

    data = sender.sign_message(encode_defunct(msg_hash)).signature
    return data


if __name__ == '__main__':
    signer = Account.from_key('0xc698ee6aa9936dac1973495bc22aceadd5dafcc1d64c439d02faf8a1ada28551')
    user_address = signer.address
    sign = get_sign_(user_address, signer)
    print(sign.hex())
