package voter_test

import (
	"testing"

	keepertest "github.com/cosmonaut/voter/testutil/keeper"
	"github.com/cosmonaut/voter/x/voter"
	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/stretchr/testify/require"
)

func TestGenesis(t *testing.T) {
	genesisState := types.GenesisState{
		RollupList: []types.Rollup{
			{
				Id: 0,
			},
			{
				Id: 1,
			},
		},
		RollupCount: 2,
		MessageList: []types.Message{
			{
				Id: 0,
			},
			{
				Id: 1,
			},
		},
		MessageCount: 2,
		L3NodeList: []types.L3Node{
			{
				Id: 0,
			},
			{
				Id: 1,
			},
		},
		L3NodeCount: 2,
		AttachedUserList: []types.AttachedUser{
			{
				IidapId: 0,
				Account: 0,
			},
			{
				IidapId: 1,
				Account: 1,
			},
		},
		// this line is used by starport scaffolding # genesis/test/state
	}

	k, ctx := keepertest.VoterKeeper(t)
	voter.InitGenesis(ctx, *k, genesisState)
	got := voter.ExportGenesis(ctx, *k)
	require.NotNil(t, got)

	require.Len(t, got.RollupList, len(genesisState.RollupList))
	require.Subset(t, genesisState.RollupList, got.RollupList)
	require.Equal(t, genesisState.RollupCount, got.RollupCount)
	require.Len(t, got.MessageList, len(genesisState.MessageList))
	require.Subset(t, genesisState.MessageList, got.MessageList)
	require.Equal(t, genesisState.MessageCount, got.MessageCount)
	require.Len(t, got.L3NodeList, len(genesisState.L3NodeList))
	require.Subset(t, genesisState.L3NodeList, got.L3NodeList)
	require.Equal(t, genesisState.L3NodeCount, got.L3NodeCount)
	require.Len(t, got.AttachedUserList, len(genesisState.AttachedUserList))
	require.Subset(t, genesisState.AttachedUserList, got.AttachedUserList)
	// this line is used by starport scaffolding # genesis/test/assert
}
