package keeper

import (
	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/store/prefix"
	sdk "github.com/cosmos/cosmos-sdk/types"
)

// SetAttachedUser set a specific attachedUser in the store from its index
func (k Keeper) SetAttachedUser(ctx sdk.Context, attachedUser types.AttachedUser) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.AttachedUserKeyPrefix))
	b := k.cdc.MustMarshal(&attachedUser)
	store.Set(types.AttachedUserKey(
		attachedUser.IidapId,
		attachedUser.Account,
	), b)
}

// GetAttachedUser returns a attachedUser from its index
func (k Keeper) GetAttachedUser(
	ctx sdk.Context,
	iidapId int32,
	account int32,

) (val types.AttachedUser, found bool) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.AttachedUserKeyPrefix))

	b := store.Get(types.AttachedUserKey(
		iidapId,
		account,
	))
	if b == nil {
		return val, false
	}

	k.cdc.MustUnmarshal(b, &val)
	return val, true
}

// RemoveAttachedUser removes a attachedUser from the store
func (k Keeper) RemoveAttachedUser(
	ctx sdk.Context,
	iidapId int32,
	account int32,

) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.AttachedUserKeyPrefix))
	store.Delete(types.AttachedUserKey(
		iidapId,
		account,
	))
}

// GetAllAttachedUser returns all attachedUser
func (k Keeper) GetAllAttachedUser(ctx sdk.Context) (list []types.AttachedUser) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.AttachedUserKeyPrefix))
	iterator := sdk.KVStorePrefixIterator(store, []byte{})

	defer iterator.Close()

	for ; iterator.Valid(); iterator.Next() {
		var val types.AttachedUser
		k.cdc.MustUnmarshal(iterator.Value(), &val)
		list = append(list, val)
	}

	return
}
