package keeper

import (
	"context"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/store/prefix"
	sdk "github.com/cosmos/cosmos-sdk/types"
	"github.com/cosmos/cosmos-sdk/types/query"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

func (k Keeper) AttachedUserAll(c context.Context, req *types.QueryAllAttachedUserRequest) (*types.QueryAllAttachedUserResponse, error) {
	if req == nil {
		return nil, status.Error(codes.InvalidArgument, "invalid request")
	}

	var attachedUsers []types.AttachedUser
	ctx := sdk.UnwrapSDKContext(c)

	store := ctx.KVStore(k.storeKey)
	attachedUserStore := prefix.NewStore(store, types.KeyPrefix(types.AttachedUserKeyPrefix))

	pageRes, err := query.Paginate(attachedUserStore, req.Pagination, func(key []byte, value []byte) error {
		var attachedUser types.AttachedUser
		if err := k.cdc.Unmarshal(value, &attachedUser); err != nil {
			return err
		}

		attachedUsers = append(attachedUsers, attachedUser)
		return nil
	})

	if err != nil {
		return nil, status.Error(codes.Internal, err.Error())
	}

	return &types.QueryAllAttachedUserResponse{AttachedUser: attachedUsers, Pagination: pageRes}, nil
}

func (k Keeper) AttachedUser(c context.Context, req *types.QueryGetAttachedUserRequest) (*types.QueryGetAttachedUserResponse, error) {
	if req == nil {
		return nil, status.Error(codes.InvalidArgument, "invalid request")
	}
	ctx := sdk.UnwrapSDKContext(c)

	val, found := k.GetAttachedUser(
		ctx,
		req.IidapId,
		req.Account,
	)
	if !found {
		return nil, status.Error(codes.InvalidArgument, "not found")
	}

	return &types.QueryGetAttachedUserResponse{AttachedUser: val}, nil
}
