package keeper

import (
	"context"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/store/prefix"
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
	"github.com/cosmos/cosmos-sdk/types/query"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"
)

func (k Keeper) RollupAll(c context.Context, req *types.QueryAllRollupRequest) (*types.QueryAllRollupResponse, error) {
	if req == nil {
		return nil, status.Error(codes.InvalidArgument, "invalid request")
	}

	var rollups []types.Rollup
	ctx := sdk.UnwrapSDKContext(c)

	store := ctx.KVStore(k.storeKey)
	rollupStore := prefix.NewStore(store, types.KeyPrefix(types.RollupKey))

	pageRes, err := query.Paginate(rollupStore, req.Pagination, func(key []byte, value []byte) error {
		var rollup types.Rollup
		if err := k.cdc.Unmarshal(value, &rollup); err != nil {
			return err
		}

		rollups = append(rollups, rollup)
		return nil
	})

	if err != nil {
		return nil, status.Error(codes.Internal, err.Error())
	}

	return &types.QueryAllRollupResponse{Rollup: rollups, Pagination: pageRes}, nil
}

func (k Keeper) Rollup(c context.Context, req *types.QueryGetRollupRequest) (*types.QueryGetRollupResponse, error) {
	if req == nil {
		return nil, status.Error(codes.InvalidArgument, "invalid request")
	}

	ctx := sdk.UnwrapSDKContext(c)
	rollup, found := k.GetRollup(ctx, req.Id)
	if !found {
		return nil, sdkerrors.ErrKeyNotFound
	}

	return &types.QueryGetRollupResponse{Rollup: rollup}, nil
}
