package keeper

import (
	"context"

	"github.com/cosmonaut/voter/x/voter/types"
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

func (k msgServer) CreateAttachedUser(goCtx context.Context, msg *types.MsgCreateAttachedUser) (*types.MsgCreateAttachedUserResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	// Check if the value already exists
	_, isFound := k.GetAttachedUser(
		ctx,
		msg.IidapId,
		msg.Account,
	)
	if isFound {
		return nil, sdkerrors.Wrap(sdkerrors.ErrInvalidRequest, "index already set")
	}

	var attachedUser = types.AttachedUser{
		Creator:  msg.Creator,
		IidapId:  msg.IidapId,
		Account:  msg.Account,
		UserName: msg.UserName,
	}

	k.SetAttachedUser(
		ctx,
		attachedUser,
	)
	return &types.MsgCreateAttachedUserResponse{}, nil
}

func (k msgServer) UpdateAttachedUser(goCtx context.Context, msg *types.MsgUpdateAttachedUser) (*types.MsgUpdateAttachedUserResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	// Check if the value exists
	valFound, isFound := k.GetAttachedUser(
		ctx,
		msg.IidapId,
		msg.Account,
	)
	if !isFound {
		return nil, sdkerrors.Wrap(sdkerrors.ErrKeyNotFound, "index not set")
	}

	// Checks if the the msg creator is the same as the current owner
	if msg.Creator != valFound.Creator {
		return nil, sdkerrors.Wrap(sdkerrors.ErrUnauthorized, "incorrect owner")
	}

	var attachedUser = types.AttachedUser{
		Creator:  msg.Creator,
		IidapId:  msg.IidapId,
		Account:  msg.Account,
		UserName: msg.UserName,
	}

	k.SetAttachedUser(ctx, attachedUser)

	return &types.MsgUpdateAttachedUserResponse{}, nil
}

func (k msgServer) DeleteAttachedUser(goCtx context.Context, msg *types.MsgDeleteAttachedUser) (*types.MsgDeleteAttachedUserResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	// Check if the value exists
	valFound, isFound := k.GetAttachedUser(
		ctx,
		msg.IidapId,
		msg.Account,
	)
	if !isFound {
		return nil, sdkerrors.Wrap(sdkerrors.ErrKeyNotFound, "index not set")
	}

	// Checks if the the msg creator is the same as the current owner
	if msg.Creator != valFound.Creator {
		return nil, sdkerrors.Wrap(sdkerrors.ErrUnauthorized, "incorrect owner")
	}

	k.RemoveAttachedUser(
		ctx,
		msg.IidapId,
		msg.Account,
	)

	return &types.MsgDeleteAttachedUserResponse{}, nil
}
