package keeper

import (
	"context"
	"fmt"

	"github.com/cosmonaut/voter/x/voter/types"
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

func (k msgServer) CreateL3Node(goCtx context.Context, msg *types.MsgCreateL3Node) (*types.MsgCreateL3NodeResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	var l3Node = types.L3Node{
		Creator:   msg.Creator,
		PublicKey: msg.PublicKey,
		Options:   msg.Options,
	}

	id := k.AppendL3Node(
		ctx,
		l3Node,
	)

	return &types.MsgCreateL3NodeResponse{
		Id: id,
	}, nil
}

func (k msgServer) UpdateL3Node(goCtx context.Context, msg *types.MsgUpdateL3Node) (*types.MsgUpdateL3NodeResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	var l3Node = types.L3Node{
		Creator:   msg.Creator,
		Id:        msg.Id,
		PublicKey: msg.PublicKey,
		Options:   msg.Options,
	}

	// Checks that the element exists
	val, found := k.GetL3Node(ctx, msg.Id)
	if !found {
		return nil, sdkerrors.Wrap(sdkerrors.ErrKeyNotFound, fmt.Sprintf("key %d doesn't exist", msg.Id))
	}

	// Checks if the msg creator is the same as the current owner
	if msg.Creator != val.Creator {
		return nil, sdkerrors.Wrap(sdkerrors.ErrUnauthorized, "incorrect owner")
	}

	k.SetL3Node(ctx, l3Node)

	return &types.MsgUpdateL3NodeResponse{}, nil
}

func (k msgServer) DeleteL3Node(goCtx context.Context, msg *types.MsgDeleteL3Node) (*types.MsgDeleteL3NodeResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	// Checks that the element exists
	val, found := k.GetL3Node(ctx, msg.Id)
	if !found {
		return nil, sdkerrors.Wrap(sdkerrors.ErrKeyNotFound, fmt.Sprintf("key %d doesn't exist", msg.Id))
	}

	// Checks if the msg creator is the same as the current owner
	if msg.Creator != val.Creator {
		return nil, sdkerrors.Wrap(sdkerrors.ErrUnauthorized, "incorrect owner")
	}

	k.RemoveL3Node(ctx, msg.Id)

	return &types.MsgDeleteL3NodeResponse{}, nil
}
