package keeper

import (
	"context"
	"fmt"

	"github.com/cosmonaut/voter/x/voter/types"
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

func (k msgServer) CreateRollup(goCtx context.Context, msg *types.MsgCreateRollup) (*types.MsgCreateRollupResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	var rollup = types.Rollup{
		Creator:     msg.Creator,
		PrevHash:    msg.PrevHash,
		CurrentHash: msg.CurrentHash,
		DsUri:       msg.DsUri,
		Inbox:       msg.Inbox,
	}

	id := k.AppendRollup(
		ctx,
		rollup,
	)

	return &types.MsgCreateRollupResponse{
		Id: id,
	}, nil
}

func (k msgServer) UpdateRollup(goCtx context.Context, msg *types.MsgUpdateRollup) (*types.MsgUpdateRollupResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	var rollup = types.Rollup{
		Creator:     msg.Creator,
		Id:          msg.Id,
		PrevHash:    msg.PrevHash,
		CurrentHash: msg.CurrentHash,
		DsUri:       msg.DsUri,
		Inbox:       msg.Inbox,
	}

	// Checks that the element exists
	val, found := k.GetRollup(ctx, msg.Id)
	if !found {
		return nil, sdkerrors.Wrap(sdkerrors.ErrKeyNotFound, fmt.Sprintf("key %d doesn't exist", msg.Id))
	}

	// Checks if the msg creator is the same as the current owner
	if msg.Creator != val.Creator {
		return nil, sdkerrors.Wrap(sdkerrors.ErrUnauthorized, "incorrect owner")
	}

	k.SetRollup(ctx, rollup)

	return &types.MsgUpdateRollupResponse{}, nil
}

func (k msgServer) DeleteRollup(goCtx context.Context, msg *types.MsgDeleteRollup) (*types.MsgDeleteRollupResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	// Checks that the element exists
	val, found := k.GetRollup(ctx, msg.Id)
	if !found {
		return nil, sdkerrors.Wrap(sdkerrors.ErrKeyNotFound, fmt.Sprintf("key %d doesn't exist", msg.Id))
	}

	// Checks if the msg creator is the same as the current owner
	if msg.Creator != val.Creator {
		return nil, sdkerrors.Wrap(sdkerrors.ErrUnauthorized, "incorrect owner")
	}

	k.RemoveRollup(ctx, msg.Id)

	return &types.MsgDeleteRollupResponse{}, nil
}
