package keeper_test

import (
	"testing"

	keepertest "github.com/cosmonaut/voter/testutil/keeper"
	"github.com/cosmonaut/voter/x/voter/keeper"
	"github.com/cosmonaut/voter/x/voter/types"
	sdk "github.com/cosmos/cosmos-sdk/types"
	"github.com/stretchr/testify/require"
)

func createNRollup(keeper *keeper.Keeper, ctx sdk.Context, n int) []types.Rollup {
	items := make([]types.Rollup, n)
	for i := range items {
		items[i].Id = keeper.AppendRollup(ctx, items[i])
	}
	return items
}

func TestRollupGet(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNRollup(keeper, ctx, 10)
	for _, item := range items {
		got, found := keeper.GetRollup(ctx, item.Id)
		require.True(t, found)
		require.Equal(t, item, got)
	}
}

func TestRollupRemove(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNRollup(keeper, ctx, 10)
	for _, item := range items {
		keeper.RemoveRollup(ctx, item.Id)
		_, found := keeper.GetRollup(ctx, item.Id)
		require.False(t, found)
	}
}

func TestRollupGetAll(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNRollup(keeper, ctx, 10)
	require.ElementsMatch(t, items, keeper.GetAllRollup(ctx))
}

func TestRollupCount(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	items := createNRollup(keeper, ctx, 10)
	count := uint64(len(items))
	require.Equal(t, count, keeper.GetRollupCount(ctx))
}
