package types

import (
	"fmt"
)

// DefaultIndex is the default capability global index
const DefaultIndex uint64 = 1

// DefaultGenesis returns the default Capability genesis state
func DefaultGenesis() *GenesisState {
	return &GenesisState{
		RollupList:       []Rollup{},
		MessageList:      []Message{},
		L3NodeList:       []L3Node{},
		AttachedUserList: []AttachedUser{},
		// this line is used by starport scaffolding # genesis/types/default
	}
}

// Validate performs basic genesis state validation returning an error upon any
// failure.
func (gs GenesisState) Validate() error {
	// Check for duplicated ID in rollup
	rollupIdMap := make(map[uint64]bool)
	rollupCount := gs.GetRollupCount()
	for _, elem := range gs.RollupList {
		if _, ok := rollupIdMap[elem.Id]; ok {
			return fmt.Errorf("duplicated id for rollup")
		}
		if elem.Id >= rollupCount {
			return fmt.Errorf("rollup id should be lower or equal than the last id")
		}
		rollupIdMap[elem.Id] = true
	}
	// Check for duplicated ID in message
	messageIdMap := make(map[uint64]bool)
	messageCount := gs.GetMessageCount()
	for _, elem := range gs.MessageList {
		if _, ok := messageIdMap[elem.Id]; ok {
			return fmt.Errorf("duplicated id for message")
		}
		if elem.Id >= messageCount {
			return fmt.Errorf("message id should be lower or equal than the last id")
		}
		messageIdMap[elem.Id] = true
	}
	// Check for duplicated ID in l3Node
	l3NodeIdMap := make(map[uint64]bool)
	l3NodeCount := gs.GetL3NodeCount()
	for _, elem := range gs.L3NodeList {
		if _, ok := l3NodeIdMap[elem.Id]; ok {
			return fmt.Errorf("duplicated id for l3Node")
		}
		if elem.Id >= l3NodeCount {
			return fmt.Errorf("l3Node id should be lower or equal than the last id")
		}
		l3NodeIdMap[elem.Id] = true
	}
	// Check for duplicated index in attachedUser
	attachedUserIndexMap := make(map[string]struct{})

	for _, elem := range gs.AttachedUserList {
		index := string(AttachedUserKey(elem.IidapId, elem.Account))
		if _, ok := attachedUserIndexMap[index]; ok {
			return fmt.Errorf("duplicated index for attachedUser")
		}
		attachedUserIndexMap[index] = struct{}{}
	}
	// this line is used by starport scaffolding # genesis/types/validate

	return nil
}
