package types_test

import (
	"testing"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/stretchr/testify/require"
)

func TestGenesisState_Validate(t *testing.T) {
	for _, tc := range []struct {
		desc     string
		genState *types.GenesisState
		valid    bool
	}{
		{
			desc:     "default is valid",
			genState: types.DefaultGenesis(),
			valid:    true,
		},
		{
			desc: "valid genesis state",
			genState: &types.GenesisState{
				RollupList: []types.Rollup{
					{
						Id: 0,
					},
					{
						Id: 1,
					},
				},
				RollupCount: 2,
				MessageList: []types.Message{
					{
						Id: 0,
					},
					{
						Id: 1,
					},
				},
				MessageCount: 2,
				L3NodeList: []types.L3Node{
					{
						Id: 0,
					},
					{
						Id: 1,
					},
				},
				L3NodeCount: 2,
				AttachedUserList: []types.AttachedUser{
					{
						IidapId: 0,
						Account: 0,
					},
					{
						IidapId: 1,
						Account: 1,
					},
				},
				// this line is used by starport scaffolding # types/genesis/validField
			},
			valid: true,
		},
		{
			desc: "duplicated rollup",
			genState: &types.GenesisState{
				RollupList: []types.Rollup{
					{
						Id: 0,
					},
					{
						Id: 0,
					},
				},
			},
			valid: false,
		},
		{
			desc: "invalid rollup count",
			genState: &types.GenesisState{
				RollupList: []types.Rollup{
					{
						Id: 1,
					},
				},
				RollupCount: 0,
			},
			valid: false,
		},
		{
			desc: "duplicated message",
			genState: &types.GenesisState{
				MessageList: []types.Message{
					{
						Id: 0,
					},
					{
						Id: 0,
					},
				},
			},
			valid: false,
		},
		{
			desc: "invalid message count",
			genState: &types.GenesisState{
				MessageList: []types.Message{
					{
						Id: 1,
					},
				},
				MessageCount: 0,
			},
			valid: false,
		},
		{
			desc: "duplicated l3Node",
			genState: &types.GenesisState{
				L3NodeList: []types.L3Node{
					{
						Id: 0,
					},
					{
						Id: 0,
					},
				},
			},
			valid: false,
		},
		{
			desc: "invalid l3Node count",
			genState: &types.GenesisState{
				L3NodeList: []types.L3Node{
					{
						Id: 1,
					},
				},
				L3NodeCount: 0,
			},
			valid: false,
		},
		{
			desc: "duplicated attachedUser",
			genState: &types.GenesisState{
				AttachedUserList: []types.AttachedUser{
					{
						IidapId: 0,
						Account: 0,
					},
					{
						IidapId: 0,
						Account: 0,
					},
				},
			},
			valid: false,
		},
		// this line is used by starport scaffolding # types/genesis/testcase
	} {
		t.Run(tc.desc, func(t *testing.T) {
			err := tc.genState.Validate()
			if tc.valid {
				require.NoError(t, err)
			} else {
				require.Error(t, err)
			}
		})
	}
}
