package types

import (
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

var _ sdk.Msg = &MsgCreateAttachedUser{}

func NewMsgCreateAttachedUser(
	creator string,
	iidapId int32,
	account int32,
	userName string,

) *MsgCreateAttachedUser {
	return &MsgCreateAttachedUser{
		Creator:  creator,
		IidapId:  iidapId,
		Account:  account,
		UserName: userName,
	}
}

func (msg *MsgCreateAttachedUser) Route() string {
	return RouterKey
}

func (msg *MsgCreateAttachedUser) Type() string {
	return "CreateAttachedUser"
}

func (msg *MsgCreateAttachedUser) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgCreateAttachedUser) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgCreateAttachedUser) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}

var _ sdk.Msg = &MsgUpdateAttachedUser{}

func NewMsgUpdateAttachedUser(
	creator string,
	iidapId int32,
	account int32,
	userName string,

) *MsgUpdateAttachedUser {
	return &MsgUpdateAttachedUser{
		Creator:  creator,
		IidapId:  iidapId,
		Account:  account,
		UserName: userName,
	}
}

func (msg *MsgUpdateAttachedUser) Route() string {
	return RouterKey
}

func (msg *MsgUpdateAttachedUser) Type() string {
	return "UpdateAttachedUser"
}

func (msg *MsgUpdateAttachedUser) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgUpdateAttachedUser) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgUpdateAttachedUser) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}

var _ sdk.Msg = &MsgDeleteAttachedUser{}

func NewMsgDeleteAttachedUser(
	creator string,
	iidapId int32,
	account int32,

) *MsgDeleteAttachedUser {
	return &MsgDeleteAttachedUser{
		Creator: creator,
		IidapId: iidapId,
		Account: account,
	}
}
func (msg *MsgDeleteAttachedUser) Route() string {
	return RouterKey
}

func (msg *MsgDeleteAttachedUser) Type() string {
	return "DeleteAttachedUser"
}

func (msg *MsgDeleteAttachedUser) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgDeleteAttachedUser) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgDeleteAttachedUser) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}
