package types

import (
	"testing"

	"github.com/cosmonaut/voter/testutil/sample"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
	"github.com/stretchr/testify/require"
)

func TestMsgCreateL3Node_ValidateBasic(t *testing.T) {
	tests := []struct {
		name string
		msg  MsgCreateL3Node
		err  error
	}{
		{
			name: "invalid address",
			msg: MsgCreateL3Node{
				Creator: "invalid_address",
			},
			err: sdkerrors.ErrInvalidAddress,
		}, {
			name: "valid address",
			msg: MsgCreateL3Node{
				Creator: sample.AccAddress(),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			err := tt.msg.ValidateBasic()
			if tt.err != nil {
				require.ErrorIs(t, err, tt.err)
				return
			}
			require.NoError(t, err)
		})
	}
}

func TestMsgUpdateL3Node_ValidateBasic(t *testing.T) {
	tests := []struct {
		name string
		msg  MsgUpdateL3Node
		err  error
	}{
		{
			name: "invalid address",
			msg: MsgUpdateL3Node{
				Creator: "invalid_address",
			},
			err: sdkerrors.ErrInvalidAddress,
		}, {
			name: "valid address",
			msg: MsgUpdateL3Node{
				Creator: sample.AccAddress(),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			err := tt.msg.ValidateBasic()
			if tt.err != nil {
				require.ErrorIs(t, err, tt.err)
				return
			}
			require.NoError(t, err)
		})
	}
}

func TestMsgDeleteL3Node_ValidateBasic(t *testing.T) {
	tests := []struct {
		name string
		msg  MsgDeleteL3Node
		err  error
	}{
		{
			name: "invalid address",
			msg: MsgDeleteL3Node{
				Creator: "invalid_address",
			},
			err: sdkerrors.ErrInvalidAddress,
		}, {
			name: "valid address",
			msg: MsgDeleteL3Node{
				Creator: sample.AccAddress(),
			},
		},
	}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			err := tt.msg.ValidateBasic()
			if tt.err != nil {
				require.ErrorIs(t, err, tt.err)
				return
			}
			require.NoError(t, err)
		})
	}
}
