package types

import (
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

var _ sdk.Msg = &MsgCreateRollup{}

func NewMsgCreateRollup(creator string, prevHash string, currentHash string, dsUri string, inbox []string) *MsgCreateRollup {
	return &MsgCreateRollup{
		Creator:     creator,
		PrevHash:    prevHash,
		CurrentHash: currentHash,
		DsUri:       dsUri,
		Inbox:       inbox,
	}
}

func (msg *MsgCreateRollup) Route() string {
	return RouterKey
}

func (msg *MsgCreateRollup) Type() string {
	return "CreateRollup"
}

func (msg *MsgCreateRollup) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgCreateRollup) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgCreateRollup) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}

var _ sdk.Msg = &MsgUpdateRollup{}

func NewMsgUpdateRollup(creator string, id uint64, prevHash string, currentHash string, dsUri string, inbox []string) *MsgUpdateRollup {
	return &MsgUpdateRollup{
		Id:          id,
		Creator:     creator,
		PrevHash:    prevHash,
		CurrentHash: currentHash,
		DsUri:       dsUri,
		Inbox:       inbox,
	}
}

func (msg *MsgUpdateRollup) Route() string {
	return RouterKey
}

func (msg *MsgUpdateRollup) Type() string {
	return "UpdateRollup"
}

func (msg *MsgUpdateRollup) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgUpdateRollup) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgUpdateRollup) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}

var _ sdk.Msg = &MsgDeleteRollup{}

func NewMsgDeleteRollup(creator string, id uint64) *MsgDeleteRollup {
	return &MsgDeleteRollup{
		Id:      id,
		Creator: creator,
	}
}
func (msg *MsgDeleteRollup) Route() string {
	return RouterKey
}

func (msg *MsgDeleteRollup) Type() string {
	return "DeleteRollup"
}

func (msg *MsgDeleteRollup) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgDeleteRollup) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgDeleteRollup) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}
